#ifndef __PR_QUERY_H
#define __PR_QUERY_H
/*-------------------------------------------------------------------------
 * Copyright (c) 1999-2002 Kenneth W. Sodemann (stuffle@charter.net)
 *-------------------------------------------------------------------------
 * pr_query
 *
 * Synopsis:
 *    Handles the creation of problem report selection queries.
 *
 * $Id: pr_query.h,v 1.11 2002/01/30 00:40:02 stuffle Exp $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to
 * Free Software Foundation, Inc.
 * 59 Temple Place, Suite 330 
 * Boston, MA  02111-1307  USA
 *-------------------------------------------------------------------------
 */
/*! \file pr_query.h

    \brief Functions to handle the data in the PRepS query table.
   
    PRepS has a query table that stores the queries that the user 
    can use to select which items to display.  The functions defined
    here are used to load those queries from the database, manipulate
    the criteria for the queries, and create the SQL statement 
    associated with the query.  These functions can also be used to
    build problem report queries from the user interface.

    In most cases, there are four steps to follow when using the
    query functions:
    
    \verbatim
    1. Create the query structure using one of the 'create' functions

    2. Modify the constraints on the query using the 'add' and 'clear'
       functions.

    3. Create a SQL query string from the query structure.  This query
       string is standard SQL that can then be sent to the database for
       execution.

    4. Call destroy_pr_query() to free up the memory used by the query
       structure.
    \endverbatim

    \author Kenneth W. Sodemann (stuffle@charter.net)
    $Revision: 1.11 $
    $Date: 2002/01/30 00:40:02 $
*/ 
    
#include <glib.h>
#include <libpq-fe.h>

/*
 * __BEGIN_DECLS should be used at the beginning of your declarations,
 * so that C++ compilers don't mangle their names.  Use __END_DECLS at
 * the end of C declarations. 
 */
#undef __BEGIN_DECLS
#undef __END_DECLS
#ifdef __cplusplus
# define __BEGIN_DECLS extern "C" {
# define __END_DECLS }
#else
# define __BEGIN_DECLS /* empty */
# define __END_DECLS /* empty */
#endif

/*
 * The __P macro is used to wrap function prototypes, so that compilers
 * that don't understand ANSI C prototypes still work, and ANSI C
 * compilers can issue warnings about type mismatches. 
 */
#undef __P
#if defined (__STDC__) || defined (_AIX) \
        || (defined (__mips) && defined (_SYSTYPE_SVR4)) \
                || defined(WIN32) || defined(__cplusplus)
# define __P(protos) protos
#else
# define __P(protos) ()
#endif

/*! \def PROJECT_NAME_POS
 * Define the position of the project name within the select clause of 
 * the problem report query.
 */
/*! \def STATUS_NAME_POS
 * Define the position of the status name within the select clause of 
 * the problem report query.
 */
/*! \def PROBLEM_TYPE_NAME_POS
 * Define the position of the problem type name within the select clause of 
 * the problem report query.
 */
/*! \def SEVERITY_NAME_POS
 * Define the position of the severity name within the select clause of 
 * the problem report query.
 */
/*! \def PR_TITLE_POS
 * Define the position of the problem report title within the select 
 * clause of the problem report query.
 */
/*! \def STATUS_ORDER_POS
 * Define the position of the status order flag within the select clause of 
 * the problem report query.
 */
/*! \def SEVERITY_ORDER_POS
 * Define the position of the severity order flag within the select clause of 
 * the problem report query.
 */
/*! \def PR_NUMBER_POS
 * Define the position of the problem report number within the select 
 * clause of the problem report query.
 */
/*! \def SUBMITTER_POS
 * Define the position of the submitter's login id within the select
 * clause of the problem report query.  Only the login id is returned,
 * not the full name.
 */
/*! \def RESPONSIBLE_POS
 * Define the position within the select clause of the problem report query
 * for the login ID of the person to whom the problem report is currently
 * assigned.  Only the login id is returned, not the full name.
 */
/*! \def CREATION_DATE_POS
 * Define the position of the cration date timestamp for the current problem
 * report within the select clause of the problem report query.
 */
#define PROJECT_NAME_POS        0
#define STATUS_NAME_POS         1
#define PROBLEM_TYPE_NAME_POS   2
#define SEVERITY_NAME_POS       3
#define PR_TITLE_POS            4
#define STATUS_ORDER_POS        5
#define SEVERITY_ORDER_POS      6
#define PR_NUMBER_POS           7
#define SUBMITTER_POS           8
#define RESPONSIBLE_POS         9
#define CREATION_DATE_POS      10 

__BEGIN_DECLS
/*! \typedef pr_query_struct
 *  
 *  This structure holds the data used to build the query.  The contents
 *  of this structure are not intended for public consumption, and are 
 *  thus hidden.  Use the functions documented within this library to
 *  modify the contents of this structure.
 */
typedef struct pr_query_struct pr_query_struct;

/*! \fn create_pr_query (gchar *user_id)
 
  \brief Create the basic problem report query.

  Allocate a pr_query_struct, and initialize the data to create
  a basic problem report query.  The resulting query returns 
  all problem reports for the projects that the user has access to.  
  The results of the query are, by default, sorted  by project, 
  severity, and status.   It is the caller's responsibility to destroy 
  the query struct via a call to destroy_pr_query().                 

  This funtion is designed to create a query structure that is a 
  good starting point for further refinment from the end user.
  The function is best used in 'find' dialogs, or when creating a
  new user defined query.

  \param user_id 
  login_id of the current user.  This parameter is used to limit the 
  selection to only the projects that the user has access to.

  \return A pointer to the query structure.  It is the caller's
          responsibility to destroy the query struct via a call to
          destroy_pr_query().
*/
pr_query_struct *create_pr_query __P((gchar *user_id));


/*! \fn create_pr_query_from_table (PGconn *conn, 
                                    const gchar *user_id, 
                                    gint query_pk)

 \brief Create a problem report query, initializing the structure
        with a query defined in the query table.
 
 Allocate a pr_query_struct, and initialize the data using a query
 defined in the query table.  It is the caller's responsibility
 to destroy the query struct via a call to destroy_pr_query().

 Remember that project constraints are not stored in the query table.
 By defalt, the query is run against all projects in the current database
 that the user has access to.  Use add_project_restriction() to add
 more project constraints, if needed.

 \param conn 
 The database connection to use when getting the query info.
 \param user_id   
 The login_id of the current user.  This is used to limit the selection 
 to only the projects that the user has access to.
 \param query_pk 
 The primary key for the query to be used.
 
 \return A pointer to the query structure.
*/
pr_query_struct *create_pr_query_from_table __P((PGconn      *conn,
                                                 const gchar *user_id,
                                                 gint         query_pk));


/*! \fn create_pr_query_from_table_li (const gchar *conn_str,
                                       const gchar *user_id,
                                       gint         query_pk)
 
 \brief Similar to create_pr_query_from_table(), accept this function
        does a database login rathher than using an existing connetion. 

 Allocate a pr_query_struct, and initialize the data using a query
 defined in the query table.  It is the caller's responsibility
 to destroy the query struct via a call to destroy_pr_query().

 Remember that project constraints are not stored in the query table.
 By defalt, the query is run against all projects in the current database
 that the user has access to.  Use add_project_restriction() to add
 more project constraints, if needed.
 
 This function attempts to login to the database instead of using an
 existing connection.

 \param conn     
 The connect string.  For details on the connect string syntax, see the 
 function 'PQconnectdb' in the 'libpq' section of the 'PostgreSQL 
 Programmer's Guide'.
 \param user_id   
 The login_id of the current user.  This is used to limit the selection to 
 only the projects that the user has access to.
 \param query_pk  
 The primary key for the query to be used.
 
 \return A pointer to the query structure. 
  
 \retval NULL Failed to login to the database.
 \retval non-NULL Function was successful.
*/
pr_query_struct *create_pr_query_from_table_li __P((const gchar *conn_str,
                                                    const gchar *user_id,
                                                    gint         query_pk));


/*! \fn destroy_pr_query (pr_query_struct *pr_query)

 \brief Free the memory, etc.

 Free any memory that was allocated for the pr_query_struct.

 \param pr_query 
 A pointer to the query struct that needs to be destroyed.
*/
void destroy_pr_query __P((pr_query_struct *pr_query));


/*! \fn add_project_restriction (pr_query_struct *q, gint pk)
 
 \brief Add a project constraint to the query.

 By default the query is contrained to all projects that the current user
 has access to.  If you want to contrain the projects further, use this
 function to add the projects that you want the query to be run against.

 \param pk  
 The project number of the project to restrict by.
 \param q  
 A pointer to the query to modify.
*/
void add_project_restriction __P((pr_query_struct *q, gint pk));


/*! \fn clear_project_restrictions (pr_query_struct *q)

  \brief Clear the project restriction list.  
  
  Remove any existing project constraints associated with the query.
  The query is still limited to only the projects that the user has 
  access to.

  \param q
  A pointer to the query to modify.
*/
void clear_project_restrictions __P((pr_query_struct *q));


/*! \fn add_severity_restriction (pr_query_struct *q, gint pk)
 
 \brief Add the given severity to the severity restriction list.

 By default, the query is not restricted by severity.  If you want to 
 restrict the results by severity, use this function to add severities
 you want the query restricted to. 

 \param pk
 The severity to add to the list.
 \param q 
 A pointer to the query to modify.
*/
void add_severity_restriction __P((pr_query_struct *q, gint pk));


/*! \fn clear_severity_restrictions (pr_query_struct *q)

 \brief Clear the severity restrictions for the query. 

 \param q
 A pointer to the query to modify.
*/
void clear_severity_restrictions __P((pr_query_struct *q));
 

/*! \fn add_status_restriction (pr_query_struct *q, gint pk)

 \brief Add the given status to the status restriction list.

 By default, the query is not restricted by status.  If you want to 
 restrict the results by status, use this function to add statuses
 you want the query restricted to. 

 \param pk  
 The status to add to the list.
 \param q 
 A pointer to the query to modify.
*/
void add_status_restriction __P((pr_query_struct *q, gint pk));


/*! \fn clear_status_restrictions (pr_query_struct *q)

 \brief Clear the status restrictions for the query. 
 
 \param q 
 A pointer to the query to modify.
*/
void clear_status_restrictions __P((pr_query_struct *q));


/*! \fn add_problem_type_restriction (pr_query_struct *q, gint pk)

 \brief Add the given problem type to the problem type restriction list.
 
 By default, the query is not restricted by problem type.  If you want 
 to restrict the results by problem type, use this function to add 
 the problem types you want the query restricted to. 

 \param pk 
 The problem type to add to the list.
 \param q 
 A pointer to the query to modify.
*/
void add_problem_type_restriction __P((pr_query_struct *q, gint pk));


/*! \fn clear_problem_type_restrictions (pr_query_struct *q)

 \brief Clear the problem type restrictions for the query. 
 
 \param q : pointer to the query to modify.
*/
void clear_problem_type_restrictions __P((pr_query_struct *q));


/*! \fn add_submitter_restriction (pr_query_struct *q, gchar *id)

 \brief Add the given submitter ID to the submitter restriction list.

 By default, the query is not restricted by submitter.  If you want 
 to restrict the results by submitter, use this function to add 
 the submitters you want the query restricted to. 

 \param id 
 The login ID of the submitter to add to the list.
 \param q
 A pointer to the query to modify.
*/
void add_submitter_restriction __P((pr_query_struct *q, gchar *id));


/*! \fn clear_submitter_restrictions (pr_query_struct *q)

 \brief Clear the submitter restrictions for the query. 
 
 \param q
 A pointer to the query to modify.
*/
void clear_submitter_restrictions __P((pr_query_struct *q));


/*! \fn add_responsible_restriction (pr_query_struct *q, gchar *id)
 
 \brief Add the given responsible ID to the responsible restriction list.

 By default, the query is not restricted by the person the problem report
 is assigned to.  If you want to restrict the results by the assignee, use 
 this function to add the login ids of those you want the query restricted 
 to. 
 
 \param id 
 The responsible ID to add to the list.
 \param q
 A pointer to the query to modify.
*/
void add_responsible_restriction __P((pr_query_struct *q, gchar *id));


/*! \fn clear_responsible_restrictions (pr_query_struct *q)

 \brief Clear the responsible ID restrictions for the query. 
 
 \param q
 A pointer to the query to modify.
*/
void clear_responsible_restrictions __P((pr_query_struct *q));


/*! \fn set_raw_pr_where (pr_query_struct *q, const gchar *str)
 
  \brief Set the WHERE clause manually rather than using the 'set'
         and 'clear' functions.

  Sets the query restrictions to use the raw SQL restrictions
  provided.  Use this function if the 'set' and 'clear' functions
  will not work for what you want (for example, if you want to test
  one of the parameters for NULL).
   
  The raw SQL will be appeneded to the WHERE clause of the following 
  SQL statement:

  \verbatim
  SELECT prj.name, stat.name, pt.name, sevr.name,
         pr.title, stat.order_num, sevr.order_num,
         pr.problem_num
    FROM project prj, status stat, problem_type pt,
         severity sevr, problem_report pr
   WHERE prj.project_num = pr.project_num
     AND stat.status_num = pr.status_num
     AND sevr.severity_num = pr.severity_num
     AND pt.type_num = pr.type_num
  \endverbatim
   
  \param str
  The raw SQL restriction and ordering string.
  \param q
  A pointer to the query to modify.
       
  \warning Any restrictions (except project restrictions) setup with the
           'add' functions will not apply if this funtion is used. 
*/
void set_raw_pr_where __P((pr_query_struct *q, const gchar *str));


/*! \fn clear_raw_pr_where (pr_query_struct *q)

 \brief Clear the raw WHERE clause. 

 If a raw WHERE clause has been setup via set_raw_pr_where(), it will
 be cleared by a call to this function.  If any restrictions had been 
 setup by calls to 'add' functions, they will be effective again
 after this call.

 \param q : the query to modify.
*/
void clear_raw_pr_where __P((pr_query_struct *q));


/*! \fn set_order_by (pr_query_struct *q, const gchar *str)

 \brief Set the order by clause.

 \param str 
 The order by clause.  \bDO \bNOT include the actual 'ORDER BY' in 
 the string.
 \param q 
 A pointer to the query structure.
*/
void set_order_by __P((pr_query_struct *q, const gchar *str));


/*! \fn clear_order_by (pr_query_struct *q)

 \brief Clear the order by clause.

 \param q 
 A pointer to the query structure.
*/
void clear_order_by __P((pr_query_struct *q));


/*! \fn create_query_string (const pr_query_struct *q)

 \brief Create an executable SQL query from the query structure.

 Create the query string represented by the query structure.
 The resulting GString is valid SQL that is executable by PostgreSQL.

 This function allocates a new GString.  It is the callers responsiblity
 to destroy the GString (use g_string_free()).

 \param q 
 A pointer to the query to output as a SQL statement.
 
 \return The SQL query string representing the given query.
*/
GString *create_query_string __P((const pr_query_struct *q));

__END_DECLS
#endif
